﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using RecordingLoaders;
using RecordingInterface;

namespace PnrfReader
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnFolderSelect_Click(object sender, EventArgs e)
        {
            OpenFileDialog Dialog = new OpenFileDialog();
            Dialog.Title = "Select Recording";
            Dialog.Filter = "Recording Files (pnrf)|*.pnrf";
            if (edtSourceFileName.Text != "")
            {
                if (File.Exists(edtSourceFileName.Text))
                {
                    Dialog.FileName = edtSourceFileName.Text;
                }
            }

            if (Dialog.ShowDialog(this) == DialogResult.OK)
            {
                String myFileName = Dialog.FileName;
                if (myFileName.Length != 0)
                {
                    if (File.Exists(myFileName))
                    {
                        try
                        {
                            edtSourceFileName.Text = myFileName;
                        }
                        catch
                        {
                            MessageBox.Show("Error reading File");
                        }
                    }
                }
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            if (!File.Exists(edtSourceFileName.Text))
            {
                MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                return;
            }
            ShowInfoData();
            ShowRecordingData();
            ShowTriggerInfo();
            ShowExtraTriggerInfo();
        }

        private void ShowInfoData()
        {
            listBox2.Items.Clear();
            PNRFLoader FromDisk = new PNRFLoader();

            string strRecordingFileName = edtSourceFileName.Text;
            IRecording myData = FromDisk.LoadRecording(strRecordingFileName);

            if (myData == null)
                return;
            if (myData.DataValues == null)
                return;

            listBox2.Items.Add(string.Format("D a t a V a l u e s ({0})", myData.DataValues.Count));
            for (int i = 1; i <= myData.DataValues.Count; i++)
            {
                IDataValue DataVal = myData.DataValues[i];
                listBox2.Items.Add(string.Format("Name: {0}, Value: {1} {2}, Type: {3}", DataVal.Name, DataVal.Value, DataVal.Units, DataVal.DataType));
            }

            listBox2.Items.Add(string.Empty);
            listBox2.Items.Add(string.Format("R e c o r d e r s ({0})", myData.Recorders.Count));
            for (int i = 1; i <= myData.Recorders.Count; i++)
            {
                IDataRecorder myDataRecorder = myData.Recorders[i];
                string cPhysicalName = myDataRecorder.PhysicalName;
                string cGroup = myDataRecorder.Group.Name;

                int nTriggers = 0;
                if (myDataRecorder.Triggers != null)
                {
                    nTriggers = myDataRecorder.Triggers.Count;
                }

                listBox2.Items.Add(string.Format("Recorder: {0} Group: {1} Triggers: {2}", cPhysicalName, cGroup, nTriggers));
            }
        }

        private void ShowRecordingData()
        {
            PNRFLoader FromDisk = new PNRFLoader();

            string strRecordingFileName = edtSourceFileName.Text;
            IRecording myData = FromDisk.LoadRecording(strRecordingFileName);
            lblOutput.Text = myData.Title;
            listBox1.Items.Clear();

            if (myData.Channels.Count < 1)
            {
                MessageBox.Show("No Data");
                return;
            }

            int channelIndex = SelectChannel(myData);
            if (channelIndex < 1)
                return;
            

            IDataChannel myChannel = myData.Channels[channelIndex];
            listBox1.Items.Add(string.Format("Recording: {0}", myChannel.Recording.Title));
            listBox1.Items.Add(string.Format("Recorder: {0} Channel: {1}", myChannel.Recorder.Name, myChannel.Name));
            IDataSrc myDataSrc = myChannel.get_DataSource(DataSourceSelect.DataSourceSelect_Mixed);

            double dStartTime = myDataSrc.Sweeps.StartTime;
            double dEndTime = myDataSrc.Sweeps.EndTime;

            listBox1.Items.Add(string.Format("Recording Start: {0}, End: {1}", dStartTime, dEndTime));

            listBox1.Items.Add("");
            object mySegmentsData = null;
            myDataSrc.Data(dStartTime, dEndTime, out mySegmentsData);

            if (mySegmentsData == null)
            {
                MessageBox.Show("No Data");
                return;
            }

            IDataSegments mySegments = mySegmentsData as IDataSegments;
            if (mySegments == null)
            {
                MessageBox.Show("No Segments");
                return;
            }
            if (mySegments.Count < 1)
            {
                MessageBox.Show("No Segments");
                return;
            }
            listBox1.Items.Add($"Number of data segments: {mySegments.Count}" );

            IDataSegment mySegment;
            for (int i = 1; i <= mySegments.Count; i++)
            {
                mySegment = mySegments[i];
                listBox1.Items.Add($"Segment[{i}].Starttime = {mySegment.StartTime}  Segment[{i}].Endtime = {mySegment.EndTime} number of samples= {mySegment.NumberOfSamples}");
                
            }

             mySegment = mySegments[1];

            object oRawData;
            mySegment.Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, mySegment.NumberOfSamples, 1, out oRawData);

            if (oRawData == null)
            {
                MessageBox.Show("No Raw Data");
                return;
            }

            double[] aSamples = oRawData as double[];

            double X0 = mySegment.StartTime;
            double DeltaX = mySegment.SampleInterval;
            double X, Y;

            for (int i = 0; i < aSamples.Length; i++)
            {
                X = X0 + i * DeltaX;
                Y = aSamples[i];
                listBox1.Items.Add(string.Format("{0:000}:  X = {1}, Y = {2}", i, X, Y));

                // Do not show more then 500 samples
                if (i >= 500)
                    break;
            }
        }

        private int SelectChannel(IRecording myData)
        {
            using (ChannelSelectionForm dialog = new ChannelSelectionForm())
            {
                dialog.cbxSelectChannel.Items.Clear();
                for (int i = 1; i <= myData.Channels.Count; i++)
                {
                    dialog.cbxSelectChannel.Items.Add(myData.Channels[i].Name);
                }
                if (dialog.ShowDialog(this) == DialogResult.OK)
                {
                    return dialog.cbxSelectChannel.SelectedIndex + 1;
                }

            }
            return -1;
        }

        private void ShowTriggerInfo()
        {
            PNRFLoader FromDisk = new PNRFLoader();
            string strRecordingFileName = edtSourceFileName.Text;
            IRecording myData = FromDisk.LoadRecording(strRecordingFileName);
            if (myData.Recorders.Count < 1)
                return;
            IDataRecorder myDataRecorder = myData.Recorders[1];
            if (myDataRecorder == null)
                return;
            if (myDataRecorder.Triggers == null)
                return;
            ITriggers myTriggers = myDataRecorder.Triggers;
            if (myTriggers.Count < 1)
                return;
            int nTriggerCount = myTriggers.Count;
            listBox3.Items.Clear();
            double dTime;
            TimeMarkType TriggerType;

            for (int i = 1; i <= nTriggerCount; i++)
            {
                ITimeMark myTimeMark = myTriggers[i];
                dTime = myTimeMark.Time;
                TriggerType = myTimeMark.MarkType;
                listBox3.Items.Add(string.Format("{0}: Trigger time: {1}, type: {2}", i, dTime, TriggerType));
            }
        }

        private void ShowExtraTriggerInfo()
        {
            if (!File.Exists(edtSourceFileName.Text))
            {
                MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                return;
            }

            PNRFLoader FromDisk = new PNRFLoader();

            string strRecordingFileName = edtSourceFileName.Text;
            IRecording MyData = FromDisk.LoadRecording(strRecordingFileName);
            int nTriggerCount = 0;
            ;
            listBox4.Items.Clear();

            if (MyData.Recorders.Count < 1)
            {
                MessageBox.Show("No Recorders found");
                return;
            }
            if (MyData.Recorders[1] == null)
                return;
            if (MyData.Recorders[1].Triggers == null)
                return;
            nTriggerCount = MyData.Recorders[1].Triggers.Count;

            listBox4.Items.Add("Extended triggers");
            listBox4.Items.Add("");
            for (int Rec = 1; Rec <= MyData.Recorders.Count; Rec++)
            {
                for (int Sw = 1; Sw <= nTriggerCount; Sw++)
                {
                    if (MyData.Recorders[Rec].Triggers[Sw].MarkType == TimeMarkType.TimeMarkType_TriggerAnnotation)
                    {
                        listBox4.Items.Add(string.Format("Recorder: '{0}' has Trigger: {1} time: {2} Type: {3}", MyData.Recorders[Rec].Name, Sw, MyData.Recorders[Rec].Triggers[Sw].Time,
                           MyData.Recorders[Rec].Triggers[Sw].MarkType));
                    }
                }
            }

            listBox4.Items.Add("");
            listBox4.Items.Add("Channel information");
            listBox4.Items.Add("");
            for (int Rec = 1; Rec <= MyData.Recorders.Count; Rec++)
            {
                if (MyData.Recorders[Rec].Channels != null)
                {
                    listBox4.Items.Add(string.Format("Recorder: '{0}'", MyData.Recorders[Rec].Name));
                    if (MyData.Recorders[Rec].Channels.Count == 0)
                    {
                        listBox4.Items.Add("      No data");
                    }
                    else
                    {
                        for (int i = 1; i <= MyData.Recorders[Rec].Channels.Count; i++)
                        {
                            listBox4.Items.Add(string.Format("      Channel: '{0}'", MyData.Recorders[Rec].Channels[i].Name));
                            IDataSrc mySweepData = MyData.Recorders[Rec].Channels[i].get_DataSource(DataSourceSelect.DataSourceSelect_Sweeps);
                            if (mySweepData != null)
                            {
                                if (mySweepData.Sweeps != null)
                                {
                                    // mySweepData.Sweeps.StartTime
                                    listBox4.Items.Add(string.Format("         Sweep Start: {0} End: {1}", mySweepData.Sweeps.StartTime, mySweepData.Sweeps.EndTime));

                                    for (int j = 1; j <= mySweepData.Sweeps.Count; j++)
                                    {
                                        IDataSweep mySweep = mySweepData.Sweeps[j];
                                        listBox4.Items.Add(string.Format("             Sweep[{0}]:  Start: {1}, End: {2}, Trigger Time: {3}, Trigger source {4}", j, mySweep.StartTime, mySweep.EndTime, mySweep.TriggerTime, mySweep.TriggerSource));
                                    }
                                }
                            }
                        }
                    }
                }

            }
        }
    }
}
