//
// Program: Perception RPC client demo program
// Author: Jos van Damme
// (c) 2009, HBM
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace PnrfReader
{
     /// <summary>
     /// Class used for simple XY display
     /// </summary>
     public partial class XYGraphControl : UserControl
     {
          private const int kXAxisIndent = 30;
          private const int kYAxisIndent = 30;
          private const int kScreenScale = 10;

          private GraphicsPath gpY = new GraphicsPath();
          private Matrix rotateMatrix = new Matrix();
          private Font GraphFont = new Font("Arial", 9);

          private int m_iShowPointX = 0;
          private int m_iShowPointY = 0;
          private bool m_bShowPoint = false;
          private Pen ShowPointPen = new Pen(Color.Red, 2);

          private bool m_bXLogScale = false;
          private int m_iDecadeXWidth = 250; // pixels per decade
          private int m_DecadesX = 4;
          private int m_iLogXOrig = 0;  // (= 10 exp m_iLogOrig for 0 --> 1 
          // for 1 --> 10  
          // for -1 --> 0.1

          private bool m_bYLogScale = false;
          private int m_iDecadeYWidth = 250; // pixels per decade
          private int m_DecadesY = 2;
          private int m_iLogYOrig = 0;  // (= 10 exp m_iLogOrig for 0 --> 1 
          // for 1 --> 10  
          // for -1 --> 0.1

          private bool m_bYShowGridLines = false;
          private bool m_bYShowSubGridLines = false;
          private bool m_bXShowGridLines = false;
          private bool m_bXShowSubGridLines = false;

          private struct extra_trace
          {
               public List<PointI> Points;
               public List<PointF> OrigPoints;
          }

          private const int nExtraTraces = 20;
          private extra_trace[] m_ExtraTraces = new extra_trace[nExtraTraces];

          /// <summary>
          /// Constructor of the simple XY graph
          /// </summary>
          public XYGraphControl()
          {
               // This call is required by the WinForms Form Designer.
               m_points.Add(new PointI());

               InitializeComponent();
               rotateMatrix.RotateAt(10, new Point(0, 0));

               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    m_ExtraTraces[i].Points = new List<PointI>();
                    m_ExtraTraces[i].OrigPoints = new List<PointF>();
               }

          }

          ////////
          public Point[] GetGraphPoints(List<PointI> points)
          {
               Point[] thePointsArray = new Point[points.Count];
               int i = 0;
               foreach (PointI pt in points)
               {
                    thePointsArray[i] = pt.ToPoint();
                    if (thePointsArray[i].Y < -100000)
                         thePointsArray[i].Y = -100000;
                    i++;
               }

               return thePointsArray;
          }

          public enum dotShape
          {
               Dot,
               Plus,
               Cross,
               BigPlus
          }

          private dotShape m_DotShape = dotShape.Dot;
          private int m_DotShapeExtraTraces = nExtraTraces;

          public void ShowPoint(float x, float y)
          {
               m_iShowPointX = TranslateX(x);
               m_iShowPointY = TranslateY(y);
               m_bShowPoint = true;
               Invalidate();
          }

          public void StopShowingPoint()
          {
               m_bShowPoint = false;
               Invalidate();
          }

          public dotShape DotShape
          {
               get
               {
                    return m_DotShape;
               }
               set
               {
                    m_DotShape = value;
                    RefreshDrawing();
               }
          }

          protected override void OnPaint(PaintEventArgs pe)
          {
               Graphics g = pe.Graphics;
               Pen myPen = m_Pens[0];
               myPen.Width = m_nTraceWidth;
               Rectangle rect = this.ClientRectangle;
               DrawXAxis(g);
               DrawYAxis(g);

               if (m_nNumberOfPoints > 0)
               {
                    if (m_bConnectPoints && m_nNumberOfPoints > 1)
                    {
                         g.DrawLines(myPen, GetGraphPoints(m_points));

                         for (int i = 0; i < m_ExtraTraces.Length; i++)
                         {
                              if (m_ExtraTraces[i].Points.Count > 1)
                              {
                                   Pen myExtraPen = m_Pens[(i + 1) % 10];
                                   myExtraPen.Width = m_nTraceWidth;
                                   g.DrawLines(myExtraPen, GetGraphPoints(m_ExtraTraces[i].Points));
                              }
                         }

                         if (m_DotShape == dotShape.Dot)
                              return;
                    }

                    for (int j = 0; j < m_points.Count; j++)
                    {
                         float x = ((PointI)m_points[j]).X;
                         float y = ((PointI)m_points[j]).Y;
                         DrawDotShape(g, x, y, myPen);
                    }

                    for (int i = 0; i < m_ExtraTraces.Length; i++)
                    {
                         if (m_ExtraTraces[i].Points.Count > 1)
                         {
                              Pen myExtraPen = m_Pens[(i + 1) % 10];
                              myExtraPen.Width = m_nTraceWidth;

                              int nCount = m_ExtraTraces[i].Points.Count;
                              for (int j = 0; j < nCount; j++)
                              {
                                   float x = ((PointI)m_ExtraTraces[i].Points[j]).X;
                                   float y = ((PointI)m_ExtraTraces[i].Points[j]).Y;
                                   if (i < m_DotShapeExtraTraces)
                                        DrawDotShape(g, x, y, myExtraPen);
                              }

                              g.DrawLines(myExtraPen, GetGraphPoints(m_ExtraTraces[i].Points));
                         }
                    }
               }

               if (m_bShowPoint)
               {
                    int nSize = 4;
                    //g.DrawEllipse(ShowPointPen, m_iShowPointX - nSize, m_iShowPointY - nSize, m_iShowPointX + nSize, m_iShowPointY + nSize);
                    g.DrawEllipse(ShowPointPen, (int)(m_iShowPointX - nSize / 2), (int)(m_iShowPointY - nSize / 2), nSize, nSize);
               }

          }

          private void DrawDotShape(Graphics g, float x, float y, Pen aPen)
          {
               if (y > 100000)
                    return;
               if (y < -100000)
                    return;
               switch (m_DotShape)
               {
                    case dotShape.Dot:
                         {
                              g.DrawLine(aPen, x, y, x + 1, y);
                         }
                         break;
                    case dotShape.Plus:
                         {
                              g.DrawLine(aPen, x - 1, y, x + 2, y);
                              g.DrawLine(aPen, x, y - 1, x, y + 2);
                         }
                         break;
                    case dotShape.BigPlus:
                         {
                              g.DrawLine(aPen, x - 2, y, x + 3, y);
                              g.DrawLine(aPen, x, y - 2, x, y + 3);
                         }
                         break;
                    case dotShape.Cross:
                         {
                              g.DrawLine(aPen, x - 2, y - 2, x + 3, y + 3);
                              g.DrawLine(aPen, x - 2, y + 2, x + 3, y - 3);
                         }
                         break;
               }
          }

          private int TranslateX(float x)
          {
               int tmp = 0;
               if (m_bXLogScale)
               {
                    double dTmp = Math.Log10(x) - m_iLogXOrig;
                    tmp = this.ClientRectangle.Left + kXAxisIndent + (int)(m_iDecadeXWidth * dTmp);
               }
               else
               {
                    tmp = this.ClientRectangle.Left + kXAxisIndent + (int)((float)x * kScreenScale / XTickValue) - (int)(XOrigin * kScreenScale / XTickValue);
               }
               return tmp;
          }

          private float RevTranslateX(int x)
          {
               double dXVal;
               if (m_bXLogScale)
               {
                    double dTmp = (x - this.ClientRectangle.Left - kXAxisIndent);
                    dTmp = dTmp / m_iDecadeXWidth;
                    dXVal = Math.Pow(10, dTmp + m_iLogXOrig);
               }
               else
               {
                    dXVal = (x - this.ClientRectangle.Left - kXAxisIndent + (XOrigin * kScreenScale / XTickValue)) / (kScreenScale / XTickValue);
               }
               return (float)dXVal;
          }

          private int TranslateY(float y)
          {
               int tmp = 0;
               if (m_bYLogScale)
               {
                    double dTmp = Math.Log10(y) - m_iLogYOrig;
                    tmp = this.ClientRectangle.Height - kYAxisIndent - (int)(m_iDecadeYWidth * dTmp);

               }
               else
               {
                    tmp = this.ClientRectangle.Bottom - (((int)((float)y * kScreenScale / YTickValue) + kYAxisIndent - (int)(YOrigin * kScreenScale / YTickValue)));
               }
               return tmp;
          }

          private float RevTranslateY(int y)
          {
               double dYVal;
               //y = this.ClientRectangle.Bottom - (((dYVal * kScreenScale / YTickValue) + kYAxisIndent - (YOrigin * kScreenScale / YTickValue)));
               //(((dYVal * kScreenScale / YTickValue) + kYAxisIndent - (YOrigin * kScreenScale / YTickValue))) = this.ClientRectangle.Bottom - y;
               if (m_bYLogScale)
               {
                    double dTmp = -y + this.ClientRectangle.Height - kYAxisIndent;
                    dTmp = dTmp / m_iDecadeYWidth;
                    dYVal = Math.Pow(10, dTmp + m_iLogYOrig);
               }
               else
               {
                    dYVal = (this.ClientRectangle.Bottom - y - kYAxisIndent + (YOrigin * kScreenScale / YTickValue)) / (kScreenScale / YTickValue);
               }
               return (float)dYVal;
          }

          private float UntranslateX(int x)
          {
               float tmp = 0;
               tmp = (float)(x - this.ClientRectangle.Left + kXAxisIndent) * XTickValue + XOrigin;
               return tmp;
          }
          private float UntranslateY(int y)
          {
               float tmp = 0;
               tmp = ((float)(this.ClientRectangle.Bottom - (y + kYAxisIndent))) * YTickValue + YOrigin;
               return tmp;
          }

          private int m_nCurrentIndex = 0;
          private List<PointI> m_points = new List<PointI>();
          private List<PointF> m_OrigPoints = new List<PointF>();

          public float XValue
          {
               get
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                         return (float)(UntranslateX((m_points[m_nCurrentIndex]).X) * m_fXTickValue + XOrigin);
                    else
                         return 0F;
               }

               set
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                    {
                         PointI p = (m_points[m_nCurrentIndex]);
                         p.X = TranslateX(value);
                    }
               }
          }

          public float YValue
          {
               get
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                    {
                         return (float)(UntranslateY(((PointI)m_points[m_nCurrentIndex]).Y) * m_fYTickValue + YOrigin);
                    }
                    else
                    {
                         return 0;
                    }
               }

               set
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                    {

                         PointI p = ((PointI)m_points[m_nCurrentIndex]);
                         p.Y = TranslateY(value);
                    }
               }
          }

          public Point AbsolutePoint
          {
               get
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                    {
                         return ((PointI)m_points[m_nCurrentIndex]).ToPoint();
                    }
                    else
                    {
                         Point myPoint = new Point(0, 0);
                         return myPoint;
                    }
               }

               set
               {
                    if ((m_points.Count > 0) && (m_nCurrentIndex < m_points.Count) && (m_nCurrentIndex > -1))
                    {
                         m_points[m_nCurrentIndex] = new PointI(value);
                    }
               }
          }

          public void AutoScale()
          {
               AutoScaleX();
               AutoScaleY();
          }

          public void AutoScaleX()
          {
               float dXMax = float.MinValue;
               float dXMin = float.MaxValue;

               float x;

               for (int j = 0; j < m_OrigPoints.Count; j++)
               {
                    x = ((PointF)m_OrigPoints[j]).X;
                    if (x > dXMax) dXMax = x;
                    if (x < dXMin) dXMin = x;
               }

               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    if (m_ExtraTraces[i].Points.Count > 1)
                    {
                         int nCount = m_ExtraTraces[i].Points.Count;
                         for (int j = 0; j < nCount; j++)
                         {
                              x = ((PointF)m_ExtraTraces[i].OrigPoints[j]).X;
                              if (x > dXMax) dXMax = x;
                              if (x < dXMin) dXMin = x;
                         }
                    }
               }

               double dXpixels = this.Width - kXAxisIndent - 10;

               if (dXMax != dXMin)
               {
                    //m_fXTickValue = (float) (dXpixels / (dXMax - dXMin));
                    m_fXTickValue = (float)(kScreenScale * (dXMax - dXMin) / dXpixels);

               }
               m_ptXOrigin = dXMin;

               RefreshDrawing();
               Invalidate();
          }

          public void AutoScaleY()
          {
               float dYMax = float.MinValue;
               float dYMin = float.MaxValue;

               float y;

               for (int j = 0; j < m_OrigPoints.Count; j++)
               {
                    y = ((PointF)m_OrigPoints[j]).Y;
                    if (y > dYMax) dYMax = y;
                    if (y < dYMin) dYMin = y;
               }

               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    if (m_ExtraTraces[i].Points.Count > 1)
                    {
                         int nCount = m_ExtraTraces[i].Points.Count;
                         for (int j = 0; j < nCount; j++)
                         {
                              y = ((PointF)m_ExtraTraces[i].OrigPoints[j]).Y;
                              if (y > dYMax) dYMax = y;
                              if (y < dYMin) dYMin = y;
                         }
                    }
               }

               double dYpixels = this.Height - kYAxisIndent - 10;

               if (dYMax != dYMin)
               {
                    //m_fYTickValue = (float) (dYpixels / (dYMax - dYMin));
                    m_fYTickValue = (float)(kScreenScale * (dYMax - dYMin) / dYpixels);

               }
               m_ptYOrigin = dYMin;

               RefreshDrawing();
               Invalidate();
          }

          private bool m_bShowCursorValue = true;
          public bool ShowCursorValue
          {
               get { return m_bShowCursorValue; }
               set
               {
                    m_bShowCursorValue = value;
                    lblCursor.Visible = m_bShowCursorValue;
               }
          }

          public Color CursorValueLabelColor
          {
               get
               {
                    return lblCursor.ForeColor;
               }

               set
               {
                    lblCursor.ForeColor = value;
                    Invalidate();
               }
          }

          public int CurrentIndex
          {
               get
               {
                    return m_nCurrentIndex;
               }

               set
               {
                    m_nCurrentIndex = value;
               }

          }

          private float m_ptXOrigin = 0.0f;
          private float m_ptYOrigin = 0.0f;

          public float XOrigin
          {
               get
               {
                    return m_ptXOrigin;
               }

               set
               {
                    m_ptXOrigin = value;
                    RefreshDrawing();
                    Invalidate();
               }

          }

          public float YOrigin
          {
               get
               {
                    return m_ptYOrigin;
               }

               set
               {
                    m_ptYOrigin = value;
                    RefreshDrawing();
                    Invalidate();
               }

          }

          private int m_nNumberOfPoints = 0;
          public int NumberOfPoints
          {
               get
               {
                    return m_nNumberOfPoints;
               }

               set
               {
                    m_nNumberOfPoints = value;
               }
          }

          private float m_fXTickValue = 1.0f;
          private float m_fYTickValue = 1.0f;

          public float XTickValue
          {
               get
               {
                    return m_fXTickValue;
               }

               set
               {
                    m_fXTickValue = value;
                    Invalidate();
               }
          }

          public float YTickValue
          {
               get
               {
                    return m_fYTickValue;
               }

               set
               {
                    m_fYTickValue = value;
                    Invalidate();
               }
          }

          private string m_LabelX = "";
          private string m_LabelY = "";

          public string LabelX
          {
               get
               {
                    return m_LabelX;
               }

               set
               {
                    m_LabelX = value;
                    Invalidate();
               }

          }

          public string LabelY
          {
               get
               {
                    return m_LabelY;
               }

               set
               {
                    m_LabelY = value;
                    gpY.Reset();
                    gpY.AddString(m_LabelY, new FontFamily("Arial"), (int)FontStyle.Regular, 8, new PointF(ClientRectangle.Left + 5, (ClientRectangle.Bottom + ClientRectangle.Top) / 2), new StringFormat());
                    gpY.Transform(rotateMatrix);
                    Invalidate();
               }

          }

          [Description("Use dot dot shape on extra traces\n0 All extra traces are drawn without marking the data points")]
          public int DotShapeExtraTraces
          {
               get { return m_DotShapeExtraTraces; }
               set
               {
                    m_DotShapeExtraTraces = value;
                    Invalidate();
               }
          }


          [Description("Y-Axis scale log (=True) or linear (=False)")]
          public bool YAxisLogScale
          {
               get
               {
                    return m_bYLogScale;
               }
               set
               {

                    m_bYLogScale = value;
                    Invalidate();
               }
          }

          [Description("Show horizontal gridlines")]
          public bool YShowGridLines
          {
               get
               {
                    return m_bYShowGridLines;
               }
               set
               {

                    m_bYShowGridLines = value;
                    Invalidate();
               }
          }

          [Description("Show horizontal sub gridlines")]
          public bool YShowSubGridLines
          {
               get
               {
                    return m_bYShowSubGridLines;
               }
               set
               {

                    m_bYShowSubGridLines = value;
                    Invalidate();
               }
          }

          [Description("Show vertical sub gridlines")]
          public bool XShowSubGridLines
          {
               get
               {
                    return m_bXShowSubGridLines;
               }
               set
               {

                    m_bXShowSubGridLines = value;
                    Invalidate();
               }
          }


          [Description("Show vertical gridlines")]
          public bool XShowGridLines
          {
               get
               {
                    return m_bXShowGridLines;
               }
               set
               {

                    m_bXShowGridLines = value;
                    Invalidate();
               }
          }

          [Description("X-Axis scale log (=True) or linear (=False)")]
          public bool XAxisLogScale
          {
               get
               {
                    return m_bXLogScale;
               }
               set
               {

                    m_bXLogScale = value;
                    Invalidate();
               }
          }

          [Description("X-Axis number of decades.")]
          public int XAxisDecades
          {
               get
               {
                    return m_DecadesX;
               }
               set
               {
                    m_DecadesX = value;
                    Invalidate();
               }
          }

          [Description("Y-Axis number of decades.")]
          public int YAxisDecades
          {
               get
               {
                    return m_DecadesY;
               }
               set
               {
                    m_DecadesY = value;
                    Invalidate();
               }
          }

          [Description("X-Axis Log origin where 0 --> 1,   1 --> 10,  2 --> 100, -1 --> 0.1 etc")]
          public int XAxisLogOrig
          {
               get
               {
                    return m_iLogXOrig;
               }
               set
               {
                    m_iLogXOrig = value;
                    RefreshDrawing();
                    Invalidate();
               }
          }

          [Description("Y-Axis Log origin where 0 --> 1,   1 --> 10,  2 --> 100, -1 --> 0.1 etc")]
          public int YAxisLogOrig
          {
               get
               {
                    return m_iLogYOrig;
               }
               set
               {
                    m_iLogYOrig = value;
                    RefreshDrawing();
                    Invalidate();
               }
          }

          [Description("X-Axis Width in pixels of a decade")]
          public int XAxisDecadeWidth
          {
               get { return m_iDecadeXWidth; }
               set
               {
                    m_iDecadeXWidth = value;
                    Invalidate();
               }
          }

          [Description("Y-Axis Width in pixels of a decade")]
          public int YAxisDecadeWidth
          {
               get { return m_iDecadeYWidth; }
               set
               {
                    m_iDecadeYWidth = value;
                    Invalidate();
               }
          }

          private bool m_bConnectPoints = false;
          public bool ConnectPointsFlag
          {
               get
               {
                    return m_bConnectPoints;
               }
               set
               {
                    m_bConnectPoints = value;
                    Invalidate();
               }
          }

          //private void InitializePoints()
          //{
          //     for (int i = 0; i < 100; i++)
          //     {
          //          // for now just set all the rest of the points to the last point
          //          PointI p = (PointI)m_points[i];
          //          p.X = TranslateX(0);
          //          p.Y = TranslateY(0);
          //         //				m_points[i].X = TranslateX(0);
          //          //				m_points[i].Y = TranslateY(0);
          //     }
          //
          //}

          public void AddPoint(float x, float y)
          {
               //float[] myPoint = new float[2];
               //myPoint[0] = x;
               //myPoint[1] = y;
               PointF myPoint = new PointF(x, y);
               m_OrigPoints.Add(myPoint);
               AddPointInternal(x, y);
          }

          public void AddPoint(float x, float y, int nTrace)
          {
               PointF myPoint = new PointF(x, y);
               m_ExtraTraces[nTrace - 1].OrigPoints.Add(myPoint);
               //m_OrigPoints2.Add(myPoint);
               AddPointInternal(x, y, nTrace - 1);
          }

          public void RefreshDrawing()
          {
               ResetInternal();
               foreach (PointF myPoint in m_OrigPoints)
               {
                    AddPointInternal(myPoint.X, myPoint.Y);
               }

               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    if (m_ExtraTraces[i].OrigPoints.Count > 0)
                    {
                         for (int j = 0; j < m_ExtraTraces[i].OrigPoints.Count; j++)
                         {
                              AddPointInternal(m_ExtraTraces[i].OrigPoints[j].X, m_ExtraTraces[i].OrigPoints[j].Y, i);
                         }
                    }
               }
          }

          private void AddPointInternal(float x, float y, int nTrace)
          {
               int iX = TranslateX(x);
               int iY = TranslateY(y);

               PointI myNewPoint = new PointI(iX, iY);
               m_ExtraTraces[nTrace].Points.Add(myNewPoint);
               //m_points2.Add(myNewPoint);
          }

          private void AddPointInternal(float x, float y)
          {
               m_points.Add(new PointI());
               XValue = x;
               YValue = y;
               CurrentIndex++;
               NumberOfPoints++;
          }

          public void Reset()
          {
               m_OrigPoints.Clear();
               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    m_ExtraTraces[i].OrigPoints.Clear();
               }
               ResetInternal();
          }

          private void ResetInternal()
          {
               CurrentIndex = 0;
               NumberOfPoints = 0;
               m_points.Clear();
               for (int i = 0; i < m_ExtraTraces.Length; i++)
               {
                    m_ExtraTraces[i].Points.Clear();
               }
          }

          private void DrawXAxis(Graphics g)
          {
               if (m_bXLogScale)
               {
                    DrawXLogAxis(g);
               }
               else
               {
                    DrawXLinAxis(g);
               }
          }

          private void DrawXLogAxis(Graphics g)
          {
               g.DrawString(m_LabelX, GraphFont, m_LabelTextBrush, new PointF(50, ClientRectangle.Bottom - kYAxisIndent / 2), new StringFormat());
               Font theAxisFont = new Font("Arial", 7);
               SolidBrush myBrush = new SolidBrush(m_LabelColor);
               Rectangle Rect = new Rectangle(ClientRectangle.X, ClientRectangle.Y, ClientRectangle.Width, ClientRectangle.Height);
               double Xval;
               try
               {
                    float count = XOrigin;
                    float kXAxis = kXAxisIndent;
                    for (int j = 0; j <= m_DecadesX; j++)
                    {
                         kXAxis = kXAxisIndent + j * m_iDecadeXWidth;
                         Xval = Math.Pow(10, j + m_iLogXOrig);

                         string strCalcValue = Xval.ToString();
                         g.DrawString(strCalcValue, theAxisFont, myBrush, this.ClientRectangle.Left + kXAxis, this.ClientRectangle.Bottom - (kXAxisIndent - theAxisFont.Height / 3));

                         for (int n = 0; n < 9; n++)
                         {
                              float dPos = (float)(kXAxis + m_iDecadeXWidth * Math.Log10(n + 1));

                              if (m_bXShowSubGridLines)
                              {
                                   g.DrawLine(TheSubGridLinesPen, this.ClientRectangle.Left + dPos, this.ClientRectangle.Bottom - kYAxisIndent, this.ClientRectangle.Left + dPos, this.ClientRectangle.Top);
                              }
                              else
                              {
                                   g.DrawLine(ThePen, this.ClientRectangle.Left + dPos, this.ClientRectangle.Bottom - kYAxisIndent - 3, this.ClientRectangle.Left + dPos, this.ClientRectangle.Bottom - kYAxisIndent + 3);
                              }

                              //g.DrawLine(ThePen, ClientRectangle.Left + dPos, ClientRectangle.Bottom - kYAxisIndent, ClientRectangle.Right, ClientRectangle.Bottom - kYAxisIndent);
                              if (!m_bYLogScale)
                              {
                                   int nYZero = TranslateY(0);
                                   g.DrawLine(TheZeroPen, ClientRectangle.Left + dPos, nYZero, ClientRectangle.Right, nYZero);
                              }
                         }

                         if (m_bXShowGridLines)
                         {
                              g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxis, this.ClientRectangle.Bottom - kYAxisIndent, this.ClientRectangle.Left + kXAxis, this.ClientRectangle.Top);
                         }
                         else
                         {
                              g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxis, this.ClientRectangle.Bottom - kYAxisIndent - 8, this.ClientRectangle.Left + kXAxis, this.ClientRectangle.Bottom - kYAxisIndent + 3);
                         }

                    }
               }
               catch (Exception e)
               {
                    g.DrawString(e.Message, theAxisFont, myBrush, 10, 10);
               }
               myBrush.Dispose();
               theAxisFont.Dispose();
          }

          private void DrawXLinAxis(Graphics g)
          {
               g.DrawString(m_LabelX, GraphFont, m_LabelTextBrush, new PointF(50, ClientRectangle.Bottom - kYAxisIndent / 2), new StringFormat());
               Font theAxisFont = new Font("Arial", 7);
               SolidBrush myBrush = new SolidBrush(m_LabelColor);
               try
               {
                    float count = XOrigin;
                    int tickcount = 0;
                    for (float i = kXAxisIndent; i < this.ClientRectangle.Width; i += kScreenScale)
                    {
                         if ((tickcount % 5) == 0)
                         {
                              float xcalcValue = count;
                              //						float xcalcValue = XTickValue*UntranslateX((int)i) + XOrigin;
                              string strCalcValue = xcalcValue.ToString("###.00");
                              g.DrawString(strCalcValue, theAxisFont, myBrush, i, this.ClientRectangle.Bottom - (kXAxisIndent - theAxisFont.Height / 3));
                              g.DrawLine(ThePen, this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent - 8, this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent + 3);
                         }

                         g.DrawLine(ThePen, this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent - 3, this.ClientRectangle.Left + i, this.ClientRectangle.Bottom - kYAxisIndent + 3);
                         count += XTickValue;
                         tickcount++;
                    }

                    g.DrawLine(ThePen, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Bottom - kYAxisIndent, ClientRectangle.Right, ClientRectangle.Bottom - kYAxisIndent);
                    int nYZero = TranslateY(0);
                    g.DrawLine(TheZeroPen, ClientRectangle.Left + kXAxisIndent, nYZero, ClientRectangle.Right, nYZero);
               }
               catch (Exception e)
               {
                    g.DrawString(e.Message, theAxisFont, myBrush, 10, 10);
               }
               myBrush.Dispose();
               theAxisFont.Dispose();
          }

          private void DrawYAxis(Graphics g)
          {
               if (m_bYLogScale)
               {
                    DrawYLogAxis(g);
               }
               else
               {
                    DrawYLinAxis(g);
               }
          }

          private void DrawYLogAxis(Graphics g)
          {
               g.DrawString(m_LabelX, GraphFont, m_LabelTextBrush, new PointF(50, ClientRectangle.Bottom - kYAxisIndent / 2), new StringFormat());
               Font theAxisFont = new Font("Arial", 7);
               SolidBrush myBrush = new SolidBrush(m_LabelColor);

               g.DrawLine(ThePen, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Top, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Bottom - kYAxisIndent);

               StringFormat theFormat = new StringFormat(StringFormatFlags.DirectionVertical);
               g.DrawString(m_LabelY, GraphFont, m_LabelTextBrush, new PointF(ClientRectangle.Left, ClientRectangle.Top + 100), theFormat);

               double Yval;
               int nY;
               try
               {
                    float kYAxis = kYAxisIndent;
                    for (int j = 0; j <= m_DecadesY; j++)
                    {
                         kYAxis = kYAxisIndent + j * m_iDecadeYWidth;
                         nY = (int)(this.ClientRectangle.Height - kYAxis);
                         Yval = Math.Pow(10, j + m_iLogYOrig);

                         string strCalcValue = Yval.ToString();
                         g.DrawString(strCalcValue, theAxisFont, myBrush, this.ClientRectangle.Left + kXAxisIndent - (theAxisFont.Height + 6) - 11, nY - 5);

                         for (int n = 1; n < 9; n++)
                         {
                              int nPos = nY - (int)(m_iDecadeYWidth * Math.Log10(n + 1));
                              if (m_bYShowSubGridLines)
                              {
                                   g.DrawLine(TheSubGridLinesPen, this.ClientRectangle.Left + kXAxisIndent - 3, nPos, this.ClientRectangle.Right, nPos);
                              }
                              else
                              {
                                   g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, nPos, this.ClientRectangle.Left + kXAxisIndent + 3, nPos);
                              }
                         }

                         if (m_bYShowGridLines)
                         {
                              g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, nY, this.ClientRectangle.Right, nY);
                         }
                         else
                         {
                              g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, nY, this.ClientRectangle.Left + kXAxisIndent + 8, nY);
                         }

                    }
               }
               catch (Exception e)
               {
                    g.DrawString(e.Message, theAxisFont, myBrush, 10, 10);
               }
               myBrush.Dispose();
               theAxisFont.Dispose();
          }

          private void DrawYLinAxis(Graphics g)
          {
               SolidBrush myBrush = new SolidBrush(m_LabelColor);
               Font theAxisFont = new Font("Arial", 7);
               g.DrawLine(ThePen, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Top, ClientRectangle.Left + kXAxisIndent, ClientRectangle.Bottom - kYAxisIndent);
               //			g.DrawPath(myPen, gpY);
               StringFormat theFormat = new StringFormat(StringFormatFlags.DirectionVertical);
               g.DrawString(m_LabelY, GraphFont, m_LabelTextBrush, new PointF(ClientRectangle.Left, ClientRectangle.Top + 100), theFormat);
               float count = YOrigin;
               int tickcount = 0;
               int nY;
               for (float i = this.ClientRectangle.Height - kYAxisIndent; i > 0; i -= kScreenScale)
               {
                    if ((tickcount % 5) == 0)
                    {
                         float ycalcValue = count;
                         string strCalcValue = ycalcValue.ToString("##.00");
                         //					int index = strCalcValue.IndexOf(".");
                         //					if (index != - 1 && index + 2 < strCalcValue.Length)
                         //					{
                         //							strCalcValue = strCalcValue.Substring(0, index + 2);
                         //					}


                         nY = TranslateY(ycalcValue);
                         g.DrawString(strCalcValue, theAxisFont, myBrush, this.ClientRectangle.Left + kXAxisIndent - (theAxisFont.Height + 6) - 11, nY - 5);
                         //g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, this.ClientRectangle.Bottom - (i + kYAxisIndent), this.ClientRectangle.Left + kXAxisIndent + 8, this.ClientRectangle.Bottom - (i + kYAxisIndent));
                         g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, nY, this.ClientRectangle.Left + kXAxisIndent + 8, nY);
                    }

                    //g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, this.ClientRectangle.Bottom - (i + kYAxisIndent), this.ClientRectangle.Left + kXAxisIndent + 3, this.ClientRectangle.Bottom - (i + kYAxisIndent));
                    count += YTickValue;
                    nY = TranslateY(count);
                    g.DrawLine(ThePen, this.ClientRectangle.Left + kXAxisIndent - 3, nY, this.ClientRectangle.Left + kXAxisIndent + 3, nY);
                    tickcount++;
               }
               theAxisFont.Dispose();
               myBrush.Dispose();
          }

          Pen ThePen = new Pen(Color.Blue);
          Pen TheZeroPen = new Pen(Color.DarkSlateGray);
          Pen TheSubGridLinesPen = new Pen(Color.DarkSlateGray);

          public Color SubGridLineColor
          {
               get { return TheSubGridLinesPen.Color; }

               set
               {
                    TheSubGridLinesPen.Dispose();
                    TheSubGridLinesPen = new Pen(value);
                    Invalidate();
               }
          }

          public Color LineColor
          {
               get { return ThePen.Color; }

               set
               {
                    ThePen.Dispose();
                    ThePen = new Pen(value);
                    Invalidate();
               }
          }

          private int m_nTraceWidth = 1;
          public int TraceWidth
          {
               get { return m_nTraceWidth; }
               set
               {
                    m_nTraceWidth = value;
                    Invalidate();
               }
          }

          SolidBrush m_LabelTextBrush = new SolidBrush(Color.Blue);
          public Color LabelTextColor
          {
               get
               {
                    return m_LabelTextBrush.Color;
               }

               set
               {
                    m_LabelTextBrush.Color = value;
                    Invalidate();
               }
          }


          private Color m_LabelColor = Color.WhiteSmoke;
          public Color LabelColor
          {
               get
               {
                    return m_LabelColor;
               }

               set
               {
                    m_LabelColor = value;
                    Invalidate();
               }
          }

          private Pen[] m_Pens = new Pen[10] {  new Pen(Color.Red),
									   new Pen(Color.Yellow),
                                                new Pen(Color.Magenta),
                                                new Pen(Color.Lime),
                                                new Pen(Color.Blue),
                                                new Pen(Color.Orange),
                                                new Pen(Color.Cyan),
                                                new Pen(Color.HotPink),
									   new Pen(Color.MediumAquamarine),
                                                new Pen(Color.DarkKhaki)};

          private int NumberOfPens = 10;

          public void SetPen(int num, Color aColor)
          {
               if ((num < 1) || (num > NumberOfPens))
               {
                    return;
               }

               // replace the pen
               m_Pens[num - 1] = new Pen(aColor);
          }

          public void AddPen(Color aColor)
          {
               m_Pens[NumberOfPens] = new Pen(aColor);
               NumberOfPens++;
          }

          private void XYGraphControl_SizeChanged(object sender, System.EventArgs e)
          {
               Invalidate();
          }

          private void XYGraphControl_KeyUp(object sender, KeyEventArgs e)
          {

               if (e.KeyCode == Keys.Subtract)
               {
                    if (e.Shift)
                    {
                         this.XTickValue = 2 * XTickValue;
                    }
                    else
                    {
                         this.XTickValue = XTickValue + 0.1F * XTickValue;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Add)
               {
                    if (e.Shift)
                    {
                         this.XTickValue = XTickValue / 2;
                    }
                    else
                    {
                         this.XTickValue = XTickValue - 0.1F * XTickValue;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Right)
               {
                    if (e.Shift)
                    {
                         this.XOrigin = XOrigin - 100F * XTickValue;
                    }
                    else
                    {
                         this.XOrigin = XOrigin - 10F * XTickValue;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Left)
               {
                    if (e.Shift)
                    {
                         this.XOrigin = XOrigin + 100F * XTickValue;
                    }
                    else
                    {
                         this.XOrigin = XOrigin + 10F * XTickValue;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Up)
               {
                    if (e.Shift && e.Control)
                    {
                         this.YOrigin = YOrigin + 100F * YTickValue;
                    }
                    else if (e.Control)
                    {
                         this.YOrigin = YOrigin + 10F * YTickValue;
                    }
                    else if (e.Shift)
                    {
                         this.YTickValue = 2 * YTickValue;
                    }
                    else
                    {
                         this.YTickValue = YTickValue + 0.1F * YTickValue;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Down)
               {
                    if (e.Shift && e.Control)
                    {
                         this.YOrigin = YOrigin - 100F * YTickValue;
                    }
                    else if (e.Control)
                    {
                         this.YOrigin = YOrigin - 10F * YTickValue;
                    }
                    else if (e.Shift)
                    {
                         this.YTickValue = YTickValue / 2;
                    }
                    else
                    {
                         this.YTickValue = YTickValue - 0.1F * YTickValue;
                    }
                    RefreshDrawing();

               }
               else if (e.KeyCode == Keys.Home)
               {
                    if (e.Shift && e.Control)
                    {
                         YOrigin = 0;
                    }
                    else
                    {
                         XOrigin = 0;
                    }
                    RefreshDrawing();
               }
               else if (e.KeyCode == Keys.Multiply)
               {
                    if (e.Control)
                         this.AutoScaleY();
                    else if (e.Shift)
                         this.AutoScaleX();
                    else
                         this.AutoScale();
               }

          }

          private void XYGraphControl_MouseUp(object sender, MouseEventArgs e)
          {
               if (m_bShowCursorValue)
               {
                    float x = RevTranslateX(e.X);
                    float y = RevTranslateY(e.Y);
                    lblCursor.Text = "X=" + x.ToString() + " Y=" + y.ToString();
               }
          }

          public void CopyToClipBoard()
          {
               PrepareImage();
               Clipboard.SetImage(memImage);
          }

          [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
          public static extern bool BitBlt(
              IntPtr hObject,
              int nXDest, int nYDest,
              int nWidth, int nHeight,
              IntPtr hObjSource, int nXSrc, int nYSrc, System.Int32 dwRop);

          private Bitmap memImage;
          private const Int32 SRCCOPY = 0xCC0020;

          private void PrepareImage()
          {
               Graphics graphic = this.CreateGraphics();
               Size s = this.Size;
               memImage = new Bitmap(s.Width, s.Height, graphic);
               Graphics memGraphic = Graphics.FromImage(memImage);
               IntPtr dc1 = graphic.GetHdc();
               IntPtr dc2 = memGraphic.GetHdc();
               BitBlt(dc2, 0, 0, this.ClientRectangle.Width,
               this.ClientRectangle.Height, dc1, 0, 0, SRCCOPY);
               graphic.ReleaseHdc(dc1);
               memGraphic.ReleaseHdc(dc2);
          }

          private bool m_bOldColorsAvailable = false;
          private Color m_cOldBackColor;
          private Color m_cOldForeColor;
          private Color m_cOldLabelColor;
          private Color m_cOldLabelTextColor;
          private Color m_cOldLineColor;
          private Color m_cOldSubGridLineColor;
          private Color m_cOldCursorValueLabelColor;

          public void SetBlackAndWhite()
          {
               m_bOldColorsAvailable = true;
               m_cOldBackColor = BackColor;
               m_cOldForeColor = ForeColor;
               m_cOldLabelColor = LabelColor;
               m_cOldLabelTextColor = LabelTextColor;
               m_cOldLineColor = LineColor;
               m_cOldSubGridLineColor = SubGridLineColor;
               m_cOldCursorValueLabelColor = CursorValueLabelColor;

               BackColor = System.Drawing.Color.White;
               ForeColor = System.Drawing.Color.White;
               LabelColor = System.Drawing.Color.Black;
               LabelTextColor = System.Drawing.Color.Black;
               LineColor = System.Drawing.Color.Black;
               SubGridLineColor = System.Drawing.Color.WhiteSmoke;
               CursorValueLabelColor = System.Drawing.Color.Black;
          }

          public void RestoreColors()
          {
               if (m_bOldColorsAvailable)
               {
                    m_bOldColorsAvailable = false;
                    BackColor = m_cOldBackColor;
                    ForeColor = m_cOldForeColor;
                    LabelColor = m_cOldLabelColor;
                    LabelTextColor = m_cOldLabelTextColor;
                    LineColor = m_cOldLineColor;
                    SubGridLineColor = m_cOldSubGridLineColor;
                    CursorValueLabelColor = m_cOldCursorValueLabelColor;
               }
          }

     }

     public class PointI
     {
          public int X = 0;
          public int Y = 0;


          public PointI()
          {
          }

          public PointI(Point p)
          {

               X = p.X;
               Y = p.Y;
          }

          public PointI(int x, int y)
          {
               X = x;
               Y = y;
          }

          public Point ToPoint()
          {
               return new Point(X, Y);
          }

     }

}

