﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using RecordingLoaders;
using RecordingInterface;
using System.Xml;
using System.Runtime.InteropServices.ComTypes;
using System.Runtime.InteropServices;

namespace PnrfReader
{
     public partial class Form1 : Form
     {
          public Form1()
          {
               InitializeComponent();
          }

          private void btnFolderSelect_Click(object sender, EventArgs e)
          {
               OpenFileDialog Dialog = new OpenFileDialog();
               Dialog.Title = "Select Recording";
               Dialog.Filter = "Recording Files (pnrf)|*.pnrf";
               if (edtSourceFileName.Text != "")
               {
                    if (File.Exists(edtSourceFileName.Text))
                    {
                         Dialog.FileName = edtSourceFileName.Text;
                    }
               }

               if (Dialog.ShowDialog(this) == DialogResult.OK)
               {
                    String myFileName = Dialog.FileName;
                    if (myFileName.Length != 0)
                    {
                         if (File.Exists(myFileName))
                         {
                              try
                              {
                                   edtSourceFileName.Text = myFileName;
                              }
                              catch
                              {
                                   MessageBox.Show("Error reading File");
                              }
                         }
                    }
               }
          }

          private void btnLoad_Click(object sender, EventArgs e)
          {
               if (!File.Exists(edtSourceFileName.Text))
               {
                    MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                    return;
               }
               ShowTriggerInfo();
               ShowExtraTriggerInfo();
               ShowPrePostTriggerInfo();
          }


          private void ShowTriggerInfo()
          {
               PNRFLoader FromDisk = new PNRFLoader();
               string strRecordingFileName = edtSourceFileName.Text;
               IRecording myData = FromDisk.LoadRecording(strRecordingFileName);
               if (myData.Recorders.Count < 1) return;
               IDataRecorder myDataRecorder = myData.Recorders[1];
               if (myDataRecorder == null) return;
               if (myDataRecorder.Triggers == null) return;
               ITriggers myTriggers = myDataRecorder.Triggers;
               if (myTriggers.Count < 1) return;
               int nTriggerCount = myTriggers.Count;
               listBox3.Items.Clear();
               double dTime;
               TimeMarkType TriggerType;

               for (int i = 1; i <= nTriggerCount; i++)
               {
                    ITimeMark myTimeMark = myTriggers[i];
                    dTime = myTimeMark.Time;
                    TriggerType = myTimeMark.MarkType;
                    listBox3.Items.Add(string.Format("{0}: Trigger time: {1}, type: {2}", i, dTime, TriggerType));
               }
          }

          private void ShowExtraTriggerInfo()
          {
               if (!File.Exists(edtSourceFileName.Text))
               {
                    MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                    return;
               }

               PNRFLoader FromDisk = new PNRFLoader();

               string strRecordingFileName = edtSourceFileName.Text;
               IRecording MyData = FromDisk.LoadRecording(strRecordingFileName);
               int nTriggerCount = 0; ;
               listBox4.Items.Clear();

               if (MyData.Recorders.Count < 1)
               {
                    MessageBox.Show("No Recorders found");
                    return;
               }
               if (MyData.Recorders[1] == null) return;
               if (MyData.Recorders[1].Triggers == null) return;
               nTriggerCount = MyData.Recorders[1].Triggers.Count;

               listBox4.Items.Add("Extended triggers");
               listBox4.Items.Add("");
               for (int Rec = 1; Rec <= MyData.Recorders.Count; Rec++)
               {
                    for (int Sw = 1; Sw <= nTriggerCount; Sw++)
                    {
                         if (MyData.Recorders[Rec].Triggers[Sw].MarkType == TimeMarkType.TimeMarkType_TriggerAnnotation)
                         {
                              listBox4.Items.Add(string.Format("Recorder: '{0}' has Trigger: {1} time: {2} Type: {3}", MyData.Recorders[Rec].Name, Sw, MyData.Recorders[Rec].Triggers[Sw].Time,
                                 MyData.Recorders[Rec].Triggers[Sw].MarkType));
                         }
                    }
               }

               listBox4.Items.Add("");
               listBox4.Items.Add("Channel information");
               listBox4.Items.Add("");
               for (int Rec = 1; Rec <= MyData.Recorders.Count; Rec++)
               {
                    if (MyData.Recorders[Rec].Channels != null)
                    {
                         listBox4.Items.Add(string.Format("Recorder: '{0}'", MyData.Recorders[Rec].Name));
                         if (MyData.Recorders[Rec].Channels.Count == 0)
                         {
                              listBox4.Items.Add("      No data");
                         }
                         else
                         {
                              for (int i = 1; i <= MyData.Recorders[Rec].Channels.Count; i++)
                              {
                                   IDataChannel myChannel = MyData.Recorders[Rec].Channels[i];
                                   IDataSrc myDataSrc = myChannel.get_DataSource(DataSourceSelect.DataSourceSelect_Mixed);
                                   object mySegmentsData = null;
                                   myDataSrc.Data(myDataSrc.Sweeps.StartTime, myDataSrc.Sweeps.EndTime, out mySegmentsData);
                                   IDataSegments mySegments = mySegmentsData as IDataSegments;
                                   IDataSegment mySegment = mySegments[1];
                                   double DeltaX = mySegment.SampleInterval;

                                   listBox4.Items.Add(string.Format("      Channel: '{0}'", MyData.Recorders[Rec].Channels[i].Name));
                                   IDataSrc mySweepData = MyData.Recorders[Rec].Channels[i].get_DataSource(DataSourceSelect.DataSourceSelect_Sweeps);
                                   if (mySweepData != null)
                                   {
                                        if (mySweepData.Sweeps != null)
                                        {
                                             // mySweepData.Sweeps.StartTime
                                             listBox4.Items.Add(string.Format("         Sweep Start: {0} End: {1}", mySweepData.Sweeps.StartTime, mySweepData.Sweeps.EndTime));

                                             for (int j = 1; j <= mySweepData.Sweeps.Count; j++)
                                             {
                                                  IDataSweep mySweep = mySweepData.Sweeps[j];
                                                  double dPreTriggerTime = mySweep.TriggerTime - mySweep.StartTime;
                                                  double dPreSamples = dPreTriggerTime / DeltaX;
                                                  int nPreSamples = (int)Math.Round(dPreSamples);
                                                  double dPostTriggerTime = mySweep.EndTime - mySweep.TriggerTime;
                                                  double dPostSamples = dPostTriggerTime / DeltaX;
                                                  int nPostSamples = (int)Math.Round(dPostSamples);
                                                  listBox4.Items.Add(string.Format("             Sweep[{0}]:  Start: {1}, End: {2}, Trigger Time: {3}, Trigger source {4}, Pre: {5} ({7}), Post: {6} ({8})", j, mySweep.StartTime, mySweep.EndTime, mySweep.TriggerTime, mySweep.TriggerSource, dPreTriggerTime, dPostTriggerTime, nPreSamples, nPostSamples));
                                             }
                                        }
                                   }
                              }
                         }
                    }
               }
          }

          private void ShowPrePostTriggerInfo()
          {
               if (!File.Exists(edtSourceFileName.Text))
               {
                    MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                    return;
               }

               PNRFLoader FromDisk = new PNRFLoader();

               string strRecordingFileName = edtSourceFileName.Text;
               IRecording MyData = FromDisk.LoadRecording(strRecordingFileName);
               int nTriggerCount = 0; ;
               listBox5.Items.Clear();

               if (MyData.Recorders.Count < 1)
               {
                    MessageBox.Show("No Recorders found");
                    return;
               }
               if (MyData.Recorders[1] == null) return;
               if (MyData.Recorders[1].Triggers == null) return;
               nTriggerCount = MyData.Recorders[1].Triggers.Count;

               for (int Rec = 1; Rec <= MyData.Recorders.Count; Rec++)
               {
                    if (MyData.Recorders[Rec].Channels != null)
                    {
                         listBox5.Items.Add(string.Format("Recorder: '{0}'", MyData.Recorders[Rec].Name));
                         if (MyData.Recorders[Rec].Channels.Count == 0)
                         {
                              listBox5.Items.Add("      No data");
                         }
                         else
                         {
                              for (int i = 1; i <= MyData.Recorders[Rec].Channels.Count; i++)
                              {
                                   IDataChannel myChannel = MyData.Recorders[Rec].Channels[i];
                                   IDataSrc myDataSrc = myChannel.get_DataSource(DataSourceSelect.DataSourceSelect_Mixed);
                                   object mySegmentsData = null;
                                   myDataSrc.Data(myDataSrc.Sweeps.StartTime, myDataSrc.Sweeps.EndTime, out mySegmentsData);
                                   IDataSegments mySegments = mySegmentsData as IDataSegments;
                                   IDataSegment mySegment = mySegments[1];
                                   double DeltaX = mySegment.SampleInterval;

                                   listBox5.Items.Add(string.Format("      Channel: '{0}'", MyData.Recorders[Rec].Channels[i].Name));
                                   IDataSrc mySweepData = MyData.Recorders[Rec].Channels[i].get_DataSource(DataSourceSelect.DataSourceSelect_Sweeps);
                                   if (mySweepData != null)
                                   {
                                        if (mySweepData.Sweeps != null)
                                        {
                                             // mySweepData.Sweeps.StartTime
                                             listBox5.Items.Add(string.Format("         Sweep Start: {0} End: {1}", mySweepData.Sweeps.StartTime, mySweepData.Sweeps.EndTime));
                                             List<string> myTriggerInfos = new List<string>();
                                             List<string> myPreTriggerInfos = new List<string>();
                                             for (int j = 1; j <= mySweepData.Sweeps.Count; j++)
                                             {
                                                  IDataSweep mySweep = mySweepData.Sweeps[j];
                                                  double dPreTriggerTime = mySweep.TriggerTime - mySweep.StartTime;
                                                  double dPreSamples = dPreTriggerTime / DeltaX;
                                                  int nPreSamples = (int)Math.Round(dPreSamples);

                                                  AddToList(myPreTriggerInfos, nPreSamples);
                                                  double dPostTriggerTime = mySweep.EndTime - mySweep.TriggerTime;
                                                  double dPostSamples = dPostTriggerTime / DeltaX;
                                                  int nPostSamples = (int)Math.Round(dPostSamples);
                                                  myTriggerInfos.Add(string.Format("{0};{1};{2}", j, nPreSamples, nPostSamples));
                                                  
                                             }

                                             // Get the most frequent pretrigger length
                                             int nMostFrequent = 0;
                                             int nTmpMax = 0;
                                             for (int m = 0; m < myPreTriggerInfos.Count; m++)
                                             {
                                                  string cInfo = myPreTriggerInfos[m];
                                                  string[] aInfo = cInfo.Split(';');
                                                  int nCount = int.Parse(aInfo[1]);
                                                  if (nCount > nTmpMax)
                                                  {
                                                       nTmpMax = nCount;
                                                       nMostFrequent = int.Parse(aInfo[0]);
                                                  }
                                             }

                                             if (myPreTriggerInfos.Count > 1)
                                             {
                                                  for (int m = 0; m < myPreTriggerInfos.Count; m++)
                                                  {
                                                        string cInfo = myPreTriggerInfos[m];
                                                        string[] aInfo = cInfo.Split(';');
                                                        listBox5.Items.Add(string.Format("          Pre-Trigger length in samples: '{0}' occurs '{1}' times", aInfo[0], aInfo[1]));
                                                  }
                                             }

                                             for (int m = 0; m < myTriggerInfos.Count; m++)
                                             {
                                                  string cInfo = myTriggerInfos[m];
                                                  string[] aInfo = cInfo.Split(';');
                                                  int nPreSamples = int.Parse(aInfo[1]);
                                                  if (nPreSamples != nMostFrequent)
                                                  {
                                                       listBox5.Items.Add(string.Format("           Sweep[{0}]:  Pre: {1} , Post: {2} !!!!!!!!!!! Differs !!!!!!!!", aInfo[0], aInfo[1], aInfo[2]));
                                                  }
                                                  else
                                                  {
                                                       listBox5.Items.Add(string.Format("           Sweep[{0}]:  Pre: {1} , Post: {2}", aInfo[0], aInfo[1], aInfo[2]));
                                                  }
                                             }
                                        }
                                   }
                              }
                         }
                    }
               }
          }

          private void AddToList(List<string> myPreTriggerInfos, int nPreSamples)
          {
               // item looks like: integer; integer where 1st integer is the pretrigger size, 2nd integer is the number of times found
               
               // First find if nPresamples is already in the list
               for (int i = 0; i < myPreTriggerInfos.Count; i++)
               {
                    string cInfo = myPreTriggerInfos[i];
                    string[] aInfo = cInfo.Split(';');
                    int nPreTrigger = 0;
                    if (int.TryParse(aInfo[0], out nPreTrigger))
                    {
                         if (nPreTrigger == nPreSamples)
                         {
                              int nCounter = 0;
                              int.TryParse(aInfo[1], out nCounter);
                              nCounter++;
                              myPreTriggerInfos[i] = aInfo[0] + ';' + nCounter.ToString();
                              return;
                         }
                    }
               }

               // not found add new item
               myPreTriggerInfos.Add(  nPreSamples.ToString() + "; 1");
          }

 

     }
}
