﻿using System;
using System.Windows.Forms;
using System.IO;
using RecordingLoaders;
using RecordingInterface;

namespace PnrfReader
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnFolderSelect_Click(object sender, EventArgs e)
        {
            OpenFileDialog Dialog = new OpenFileDialog();
            Dialog.Title = "Select Recording";
            Dialog.Filter = "Recording Files (pnrf)|*.pnrf";
            if (edtSourceFileName.Text != "")
            {
                if (File.Exists(edtSourceFileName.Text))
                {
                    Dialog.FileName = edtSourceFileName.Text;
                }
            }

            if (Dialog.ShowDialog(this) == DialogResult.OK)
            {
                String myFileName = Dialog.FileName;
                if (myFileName.Length != 0)
                {
                    if (File.Exists(myFileName))
                    {
                        try
                        {
                            edtSourceFileName.Text = myFileName;
                        }
                        catch
                        {
                            MessageBox.Show("Error reading File");
                        }
                    }
                }
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            if (!File.Exists(edtSourceFileName.Text))
            {
                MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                return;
            }
            ShowInfoData();
            ShowRecordingData();
        }

        private string TimeFormat(double dTime)
        {
            if (dTime < 1E-05)
            {
                return $"{dTime:E2}s";
            }
            else
              return $"{dTime:#.###}s";
        }
        private void ShowInfoData()
        {
            listBox2.Items.Clear();
            PNRFLoader FromDisk = new PNRFLoader();

            string strRecordingFileName = edtSourceFileName.Text;
            IRecording myData = FromDisk.LoadRecording(strRecordingFileName);

            if (myData == null) return;
            if (myData.DataValues == null) return;

            listBox2.Items.Add(string.Format("D a t a V a l u e s ({0})", myData.DataValues.Count));
            for (int i = 1; i <= myData.DataValues.Count; i++)
            {
                IDataValue DataVal = myData.DataValues[i];
                listBox2.Items.Add(string.Format("Name: {0}, Value: {1} {2}, Type: {3}", DataVal.Name, DataVal.Value, DataVal.Units, DataVal.DataType));
            }

            listBox2.Items.Add(string.Empty);
            listBox2.Items.Add(string.Format("R e c o r d e r s ({0})", myData.Recorders.Count));
            for (int i = 1; i <= myData.Recorders.Count; i++)
            {
                IDataRecorder myDataRecorder = myData.Recorders[i];
                string cPhysicalName = myDataRecorder.PhysicalName;
                string cGroup = myDataRecorder.Group.Name;

                int nTriggers = 0;
                if (myDataRecorder.Triggers != null)
                {
                    nTriggers = myDataRecorder.Triggers.Count;
                }

                listBox2.Items.Add(string.Format("Recorder: {0} Group: {1} Triggers: {2}", cPhysicalName, cGroup, nTriggers));
            }
        }

        private void ShowRecordingData()
        {
            PNRFLoader FromDisk = new PNRFLoader();

            string strRecordingFileName = edtSourceFileName.Text;
            IRecording myData = FromDisk.LoadRecording(strRecordingFileName);
            lblOutput.Text = myData.Title;
            listBox1.Items.Clear();

            if (myData.Channels.Count < 1)
            {
                MessageBox.Show("No Data");
                return;
            }
            listBox1.Items.Add($"Number of channels: {myData.Channels.Count}");
            listBox1.Items.Add("");
            for (int i = 1; i <= myData.Channels.Count; i++)
            {
                listBox1.Items.Add("");
                IDataChannel myChannel = myData.Channels[i];
                listBox1.Items.Add($"----Channel[{i}]: {myChannel.Name}----");

                listBox1.Items.Add($" Recording: {myChannel.Recording.Title}");
                listBox1.Items.Add($" Recorder: {myChannel.Recorder.Name} Channel: {myChannel.Name}");
                IDataSrc myDataSrc = myChannel.get_DataSource(DataSourceSelect.DataSourceSelect_Mixed);

                double dStartTime = myDataSrc.Sweeps.StartTime;
                double dEndTime = myDataSrc.Sweeps.EndTime;

                listBox1.Items.Add($" Recording Start: {TimeFormat(dStartTime)}, End: {TimeFormat(dEndTime)}");

                object mySegmentsData = null;
                myDataSrc.Data(dStartTime, dEndTime, out mySegmentsData);

                if (mySegmentsData == null)
                {
                    listBox1.Items.Add(" No Data");
                    continue;
                }

                IDataSegments mySegments = mySegmentsData as IDataSegments;
                if (mySegments == null)
                {
                    listBox1.Items.Add(" No Segments");
                    continue;
                }
                if (mySegments.Count < 1)
                {
                    listBox1.Items.Add(" No Segments");
                    continue;
                }

                for (int j = 1; j <= mySegments.Count; j++)
                {
                    IDataSegment mySegment = mySegments[j];
                    string strTxt1 = $"   Segment[{j}] starts at: {TimeFormat(mySegment.StartTime)} and ends at {TimeFormat(mySegment.EndTime)}";
                    
                    object oYRawData;
                    object oTimes;
                    mySegment.WaveformWithTimes(DataSourceResultType.DataSourceResultType_Double64, 1, mySegment.NumberOfSamples, 1, out oYRawData, out oTimes);

                    if (oTimes != null)
                    {
                        double[] aTimes = oTimes as double[];
                        int nSamples = aTimes.Length;
                        listBox1.Items.Add($" {strTxt1}, it contains {nSamples} asynchronous samples");
                        continue;
                    }

                    object oRawData;
                    mySegment.Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, mySegment.NumberOfSamples, 1, out oRawData);
                    if (oRawData == null)
                    {
                        listBox1.Items.Add("   No Raw Data");
                    }
                    else
                    {
                        double[] aData = oRawData as double[];
                        int nSamples = aData.Length;
                        listBox1.Items.Add($" {strTxt1}, it contains {nSamples} synchronous samples");
                    }
                }
            }
        }
    }
}
